@@ -35,7 +35,10 @@
 #include <unistd.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <glib.h>
+#ifndef G_OS_WIN32
 #include <sys/wait.h>
+#endif
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -77,6 +80,14 @@ static void libgnomeui_arg_callback 	(poptContext con,
 					 const struct poptOption * opt,
 					 const char * arg,
 					 void * data);
+static gboolean libgnomeui_goption_disable_crash_dialog (const gchar *option_name,
+							 const gchar *value,
+							 gpointer data,
+							 GError **error);
+static gboolean libgnomeui_goption_display (const gchar *option_name,
+					    const gchar *value,
+					    gpointer data,
+					    GError **error);
 static void libgnomeui_init_pass	(const GnomeModuleInfo *mod_info);
 static void libgnomeui_class_init	(GnomeProgramClass *klass,
 					 const GnomeModuleInfo *mod_info);
@@ -86,10 +97,11 @@ static void libgnomeui_pre_args_parse	(GnomeProgram *app,
 					 GnomeModuleInfo *mod_info);
 static void libgnomeui_post_args_parse	(GnomeProgram *app,
 					 GnomeModuleInfo *mod_info);
-static void libgnomeui_rc_parse		(GnomeProgram *program,
-					 const gchar *command);
+static void libgnomeui_rc_parse		(GnomeProgram *program);
+#ifndef G_OS_WIN32
 static void libgnomeui_segv_setup	(GnomeProgram *program,
 					 gboolean post_arg_parse);
+#endif
 
 /* Prototype for a private gnome_stock function */
 void _gnome_stock_icons_init (void);
@@ -120,6 +132,13 @@ static struct poptOption libgnomeui_options[] = {
 	{NULL, '\0', 0, NULL, 0, NULL, NULL}
 };
 
+static GOptionEntry libgnomeui_goptions[] = {
+	{ "disable-crash-dialog", '\0', G_OPTION_FLAG_NO_ARG,
+	  G_OPTION_ARG_CALLBACK, libgnomeui_goption_disable_crash_dialog,
+	  N_("Disable Crash Dialog"), NULL },
+	{ NULL }
+};
+
 const GnomeModuleInfo *
 libgnomeui_module_info_get (void)
 {
@@ -132,6 +151,16 @@ libgnomeui_module_info_get (void)
 		NULL, NULL
 	};
 
+	GOptionGroup * option_group;
+	option_group = g_option_group_new ("gnome-ui",
+					   N_("GNOME GUI Library"),
+					   N_("Show GNOME GUI options"),
+					   NULL, NULL);
+	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+	g_option_group_add_entries(option_group, libgnomeui_goptions);
+
+	module_info.expansion1 = (gpointer) option_group;
+
 	if (module_info.requirements == NULL) {
 		static GnomeModuleRequirement req[6];
 
@@ -248,7 +277,7 @@ libgnomeui_class_init (GnomeProgramClass *klass, const GnomeModuleInfo *mod_info
                 libgnomeui_set_property,
                 g_param_spec_boolean (LIBGNOMEUI_PARAM_CRASH_DIALOG, NULL, NULL,
                                       TRUE, (G_PARAM_READABLE | G_PARAM_WRITABLE |
-                                             G_PARAM_CONSTRUCT_ONLY)));
+                                      G_PARAM_CONSTRUCT)));
         cdata->display_id = gnome_program_install_property (
                 klass,
                 libgnomeui_get_property,
@@ -288,9 +317,10 @@ libgnomeui_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 		g_object_set (G_OBJECT(app),
 			      LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE,
 			      NULL);
-
+#ifndef G_OS_WIN32
         if(do_crash_dialog)
                 libgnomeui_segv_setup (app, FALSE);
+#endif
 }
 
 #ifdef HAVE_ESD
@@ -535,10 +565,10 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
         gchar *filename;
 
         gnome_type_init();
-        libgnomeui_rc_parse (program, g_get_prgname ());
-
+        libgnomeui_rc_parse (program);
+#ifndef G_OS_WIN32
         libgnomeui_segv_setup (program, TRUE);
-
+#endif
         priv = g_object_get_qdata(G_OBJECT(program), quark_gnome_program_private_libgnomeui);
         priv->constructed = TRUE;
 
@@ -588,6 +618,18 @@ libgnomeui_arg_callback(poptContext con,
         }
 }
 
+static gboolean
+libgnomeui_goption_disable_crash_dialog (const gchar *option_name,
+					 const gchar *value,
+					 gpointer data,
+					 GError **error)
+{ 
+	g_object_set (G_OBJECT (gnome_program_get ()),
+		      LIBGNOMEUI_PARAM_CRASH_DIALOG, FALSE, NULL);
+
+	return TRUE;
+}
+
 /* automagically parse all the gtkrc files for us.
  * 
  * Parse:
@@ -603,8 +645,9 @@ libgnomeui_arg_callback(poptContext con,
  * If you don't like it.. give me a good reason.
  */
 static void
-libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
+libgnomeui_rc_parse (GnomeProgram *program)
 {
+        gchar *command = g_get_prgname ();
 	gint i;
 	gint buf_len;
 	const gchar *buf = NULL;
@@ -614,7 +657,7 @@ libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
 	buf_len = strlen(command);
 	
 	for (i = 0; i < buf_len; i++) {
-		if (command[buf_len - i] == '/') {
+		if (G_IS_DIR_SEPARATOR (command[buf_len - i])) {
 			buf = &command[buf_len - i + 1];
 			break;
 		}
@@ -659,6 +702,8 @@ libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
 	}
 }
 
+#ifndef G_OS_WIN32
+
 /* crash handler */
 static void libgnomeui_segv_handle(int signum);
 
@@ -758,7 +803,7 @@ static void libgnomeui_segv_handle(int signum)
                 program = gnome_program_get();
 
 		/* Child process */
-		execl (GNOMEUISERVERDIR "/gnome_segv2", GNOMEUISERVERDIR "/gnome_segv",
+		execl (LIBGNOMEUI_SERVERDIR "/gnome_segv2", LIBGNOMEUI_SERVERDIR "/gnome_segv",
 		       g_get_prgname (), buf,
 		       gnome_program_get_app_version (program), NULL);
 
@@ -772,6 +817,8 @@ static void libgnomeui_segv_handle(int signum)
 	in_segv--;
 }
 
+#endif
+
 /* #warning "Solve the sound events situation" */
 
 /* backwards compat */
